# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import numpy as np
from tqdm import tqdm
import itertools
import math

from ..functions.lst import type_lst
from ..functions.table import extract_int, get_col_val_lst, get_val_col, order_rows
from ..functions.col import phi_col, psi_col, dih_col_lst
from ..functions.path import save_matrix


def calc_flip_dist(norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=False):

    if norm_dist <= 2:
        flip_dist = norm_dist
    else:
        flip_dist_lst = list()
        bb_resid_lst = resid_dict[phi_col]
        min_resid = min(bb_resid_lst)
        max_resid = max(bb_resid_lst)
        for flip_diff in list(flip_dict.keys()):
            for index, resid in enumerate(bb_resid_lst):
                if resid != min_resid and resid != max_resid:
                    curr_resid = resid
                    next_resid = bb_resid_lst[index + 1]
                    if abs(extract_int(next_resid) - extract_int(curr_resid)) < 2:
                        if curr_resid in list(
                            index_dict[psi_col].keys()
                        ) and next_resid in list(index_dict[phi_col].keys()):
                            curr_index = index_dict[psi_col][curr_resid]
                            next_index = index_dict[phi_col][next_resid]

                            temp_psi_vals = np.copy(norm_dict[psi_col])
                            temp_phi_vals = np.copy(norm_dict[phi_col])

                            temp_psi_vals[curr_index] = flip_dict[flip_diff][psi_col][
                                curr_index
                            ]
                            temp_phi_vals[next_index] = flip_dict[flip_diff][phi_col][
                                next_index
                            ]

                            temp_flip_vals = np.array([])

                            temp_flip_vals = np.concatenate(
                                (temp_flip_vals, temp_psi_vals), axis=0
                            )
                            temp_flip_vals = np.concatenate(
                                (temp_flip_vals, temp_phi_vals), axis=0
                            )

                            for dih_col in dih_col_lst:
                                if dih_col != psi_col and dih_col != phi_col:
                                    norm_angle_vals = norm_dict[dih_col]
                                    if len(norm_angle_vals) > 0:
                                        temp_flip_vals = np.concatenate(
                                            (temp_flip_vals, norm_angle_vals), axis=0
                                        )
                            if mean:
                                temp_flip_dist = np.mean(temp_flip_vals)
                            else:
                                temp_flip_dist = np.max(temp_flip_vals)
                            flip_dist_lst.append(temp_flip_dist)

        if len(flip_dist_lst) == 0:
            flip_dist = norm_dist
        else:
            flip_dist = min(min(flip_dist_lst), norm_dist)

    return flip_dist


def calc_dih_dist(i, j, i_df, j_df, flip_diff=180):

    resid_dict = dict()

    for dih_col in dih_col_lst:

        resid_dict[dih_col] = get_col_val_lst(i_df, dih_col)

    norm_dict = dict()
    flip_dict = dict()

    for dih_col in dih_col_lst:

        norm_dict[dih_col] = np.array([])

    flip_diff_lst = type_lst(flip_diff)

    for flip_diff in flip_diff_lst:
        flip_dict[flip_diff] = {phi_col: np.array([]), psi_col: np.array([])}

    index_dict = dict()

    for dih_col in dih_col_lst:

        resid_lst = resid_dict[dih_col]

        index_dict[dih_col] = dict()
        index = 0

        for resid in resid_lst:

            val_col = get_val_col(dih_col, resid)

            norm_val_i = float(i_df.at[i, val_col])
            norm_val_j = float(j_df.at[j, val_col])

            if norm_val_i != 999.00 and norm_val_j != 999.00:

                norm_dist = 2 * (
                    1 - math.cos((math.radians(norm_val_i) - math.radians(norm_val_j)))
                )

                norm_dict[dih_col] = np.append(norm_dict[dih_col], norm_dist)

                for flip_diff in flip_diff_lst:

                    if dih_col == psi_col or dih_col == phi_col:
                        flip_dist = 2 * (
                            1
                            - math.cos(
                                (
                                    math.radians(norm_val_i + flip_diff)
                                    - math.radians(norm_val_j)
                                )
                            )
                        )

                        flip_dict[flip_diff][dih_col] = np.append(
                            flip_dict[flip_diff][dih_col], flip_dist
                        )

                index_dict[dih_col][resid] = index
                index += 1

    norm_vals = np.array([])

    for dih_col in dih_col_lst:
        norm_vals = np.concatenate((norm_vals, norm_dict[dih_col]), axis=0)

    if len(norm_vals) == 0:
        max_norm_dist = 4.0
        mean_norm_dist = 4.0
        max_flip_dist = 4.0
        mean_flip_dist = 4.0
    else:
        max_norm_dist = np.max(norm_vals)
        mean_norm_dist = np.mean(norm_vals)
        max_flip_dist = calc_flip_dist(
            max_norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=False
        )
        mean_flip_dist = calc_flip_dist(
            mean_norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=True
        )

    max_norm_result = (i, j, max_norm_dist)
    mean_norm_result = (i, j, mean_norm_dist)
    max_flip_result = (i, j, max_flip_dist)
    mean_flip_result = (i, j, mean_flip_dist)

    return (max_norm_result, mean_norm_result, max_flip_result, mean_flip_result)


def build_dih_matrix(
    fit_df,
    max_norm_path,
    mean_norm_path=None,
    max_flip_path=None,
    mean_flip_path=None,
    pred_df=None,
    flip_diff=180,
):

    fit_df = order_rows(fit_df)
    j_df = fit_df.copy(deep=True)

    if pred_df is None:
        i_df = fit_df.copy(deep=True)
    else:
        pred_df = order_rows(pred_df)
        i_df = pred_df.copy(deep=True)

    i_index_lst = list(i_df.index.values)
    j_index_lst = list(j_df.index.values)

    if pred_df is not None:
        index_pairs = itertools.product(i_index_lst, j_index_lst)
    else:
        index_pairs = itertools.combinations(i_index_lst, 2)

    max_norm_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    mean_norm_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    max_flip_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    mean_flip_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))

    matrix_lst = [max_norm_matrix, mean_norm_matrix, max_flip_matrix, mean_flip_matrix]

    for i_index, j_index in tqdm(
        index_pairs, desc="Building dihedral matrix", position=0, leave=True
    ):
        dih_result = calc_dih_dist(
            i_index,
            j_index,
            i_df,
            j_df,
            flip_diff=flip_diff,
        )

        result_lst = [
            dih_result[0],
            dih_result[1],
            dih_result[2],
            dih_result[3],
        ]

        for result, matrix in zip(result_lst, matrix_lst):

            i_index = result[0]
            j_index = result[1]
            dist = result[2]

            matrix[i_index, j_index] = dist

            if pred_df is None:
                matrix[j_index, i_index] = dist

    save_matrix(max_norm_path, max_norm_matrix)

    if mean_norm_path is not None:
        save_matrix(mean_norm_path, mean_norm_matrix)

    if max_flip_path is not None:
        save_matrix(max_flip_path, max_flip_matrix)

    if mean_flip_path is not None:
        save_matrix(mean_flip_path, mean_flip_matrix)

    print("Built dihedral matrix!")



# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import streamlit as st

from ..functions.table import mask_equal
from ..functions.col import pdb_code_col
from ..functions.gui import load_st_table, write_st_end, create_st_button, show_st_structure


def home_page():

    left_col, right_col = st.columns(2)

    df = load_st_table(__file__)

    show_st_structure(mask_equal(df, pdb_code_col, "6oim"),
            zoom=1.2,
            width=400,
            height=300,
            surface_trans=1,
            spin_on=True,
            st_col=left_col)

    right_col.markdown("# Rascore")
    right_col.markdown("### A tool for analyzing RAS protein structures")
    right_col.markdown("**Created by Mitchell Parker and Roland Dunbrack**")
    right_col.markdown("**Fox Chase Cancer Center**")

    database_link_dict = {
        "bioRxiv Paper": "https://www.biorxiv.org/content/10.1101/2022.02.02.478568v2",
        "GitHub Page": "https://github.com/mitch-parker/rascore",
        "RCSB Protein Data Bank": "https://www.rcsb.org",
    }

    st.sidebar.markdown("## Database-Related Links")
    for link_text, link_url in database_link_dict.items():
        create_st_button(link_text, link_url, st_col=st.sidebar)

    community_link_dict = {
        "NCI RAS Initiative": "https://www.cancer.gov/research/key-initiatives/ras",
        "KRAS Kickers": "https://www.kraskickers.org",
        "RASopathies Network": "https://rasopathiesnet.org",
    }

    st.sidebar.markdown("## Community-Related Links")
    for link_text, link_url in community_link_dict.items():
        create_st_button(link_text, link_url, st_col=st.sidebar)

    software_link_dict = {
        "BioPython": "https://biopython.org",
        "RDKit": "https://www.rdkit.org",
        "PDBrenum": "http://dunbrack.fccc.edu/PDBrenum/",
        "Fpocket": "https://bioserv.rpbs.univ-paris-diderot.fr/services/fpocket/",
        "PyMOL": "https://pymol.org/2/",
        "3Dmol": "https://3dmol.csb.pitt.edu",
        "Pandas": "https://pandas.pydata.org",
        "NumPy": "https://numpy.org",
        "SciPy": "https://scipy.org",
        "Sklearn": "https://scikit-learn.org/stable/",
        "Matplotlib": "https://matplotlib.org",
        "Seaborn": "https://seaborn.pydata.org",
        "Streamlit": "https://streamlit.io",
    }

    st.sidebar.markdown("## Software-Related Links")
    link_1_col, link_2_col, link_3_col = st.sidebar.columns(3)

    i = 0
    link_col_dict = {0: link_1_col, 1: link_2_col, 2: link_3_col}
    for link_text, link_url in software_link_dict.items():

        st_col = link_col_dict[i]
        i += 1
        if i == len(link_col_dict.keys()):
            i = 0

        create_st_button(link_text, link_url, st_col=st_col)

    st.markdown("---")

    st.markdown(
        """
        ### Summary
        *Rascore* is a tool for analyzing structures of the RAS protein family
        (KRAS, NRAS, and HRAS). The *Rascore* 
        database presents a continually updated analysis of all available
        RAS structures in the Protein Data Bank (PDB) with their catalytic SW1 and 
        SW2 loops conformationally classified and their molecular 
        contents annotated (e.g., mutation status, nucleotide state, 
        bound protein, inhibitor site). 

        Details of our work are 
        provided on [bioRxiv](https://www.biorxiv.org/content/10.1101/2022.02.02.478568v2) in the manuscript, 
        **Defining An Expanded RAS Conformational Landscape Based on Over 700 Experimentally 
        Determined Structures of KRAS, NRAS, and HRAS.** We hope that researchers will use 
        *Rascore* to gain novel insights into RAS biology and 
        drug discovery. 
        """
    )

    st.markdown(
        """
        ### Usage

        To the left, is a dropdown main menu for navigating to 
        each page in the *Rascore* database:

        - **Home Page:** We are here!
        - **Database Overview:** Overview of the *Rascore* database, molecular annotations, 
        and RAS conformational classification.
        - **Search PDB:** Search for individual PDB entries containing RAS structures.
        - **Explore Conformations:** Explore RAS SW1 and SW2 conformations found in the PDB by nucleotide state.
        - **Analyze Mutations:** Analyze the structural impact of RAS mutations by comparing WT and mutated structures.
        - **Compare Inhibitors:** Compare inhibitor-bound RAS structures by compound binding site and chemical substructure.
        - **Query Database:** Query the *Rascore* database by conformations and molecular annotations.
        - **Classify Structures:** Conformationally classify and annotate the molecular contents of uploaded RAS structures.
        """
    )
    st.markdown("---")

    st.markdown(
        f"""
        ### Authors
        Please feel free to contact us with any issues, comments, or questions.

        ##### Mitchell Parker [![Twitter URL](https://img.shields.io/twitter/url/https/twitter.com/bukotsunikki.svg?style=social&label=Follow%20%40Mitch_P)](https://twitter.com/Mitch_P)

        - Email:  <mip34@drexel.edu> or <mitchell.parker@fccc.edu>
        - GitHub: https://github.com/mitch-parker

        ##### Roland Dunbrack [![Twitter URL](https://img.shields.io/twitter/url/https/twitter.com/bukotsunikki.svg?style=social&label=Follow%20%40RolandDunbrack)](https://twitter.com/RolandDunbrack)

        - Email: <roland.dunbrack@fccc.edu>
        - GitHub: https://github.com/DunbrackLab
        """,
        unsafe_allow_html=True,
    )

    st.markdown("---")

    st.markdown(
        """
        ### Funding

        - NIH F30 GM142263 (to M.P.)
        - NIH R35 GM122517 (to R.D.)
         """
    )

    st.markdown(
        """
        ### License
        Apache License 2.0
        """
    )

    write_st_end()



# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import pandas as pd
import streamlit as st

from ..constants.conf import (
    loop_resid_dict,
    sw1_name,
    sw2_name,
    outlier_name,
    disorder_name,
    conf_color_dict,
)
from ..constants.nuc import nuc_color_dict, nuc_class_lst, gtp_name

from ..functions.gui import (
    get_neighbor_path,
    load_st_table,
    get_html_text,
    write_st_end,
    show_st_structure,
    rename_st_cols,
    show_st_table,
    reorder_st_cols
)
from ..functions.path import (
    get_file_path,
    load_table,
    pages_str,
    data_str,
    rascore_str,
    cluster_str,
)
from ..functions.table import make_dict
from ..functions.file import result_table_file
from ..functions.col import (
    rename_col_dict,
    nuc_class_col,
    gene_class_col,
    match_class_col,
    prot_class_col,
    pdb_id_col,
    complete_col,
    bio_lig_col,
)
from ..functions.table import (
    mask_equal,
    get_col_most_common,
    lst_col,
    fix_col,
)

reverse_col_dict = make_dict(list(rename_col_dict.values()),list(rename_col_dict.keys()))


def conformation_page():

    df = load_st_table(__file__)

    st.markdown("## Explore Conformations")
    st.markdown("---")

    st.sidebar.markdown(
        """
    **Note.** Only structures with completely modeled loops from our original clustering are displayed.
    Tables below structures include updated counts from *Rascore* database.
    """
    )

    data_path = get_neighbor_path(__file__, pages_str, data_str)

    sum_df = (
                    pd.pivot_table(
                        data=rename_st_cols(df),
                        index=rename_col_dict[sw2_name],
                        columns=rename_col_dict[sw1_name],
                        values=rename_col_dict[pdb_id_col],
                        aggfunc="nunique",
                        margins=True,
                    )
                    .fillna("")
                )

    for col in list(sum_df.columns):
        sum_df[col] = sum_df[col].map(str)
        sum_df = fix_col(sum_df, col)

    sum_df = reorder_st_cols(sum_df,sw2_name,sw1_name)
    sum_df = sum_df.reset_index()

    sum_df = sum_df.rename(columns={rename_col_dict[sw2_name]:'Conformation Label'})

    show_st_table(sum_df)

    sw1_df = load_table(
        get_file_path(
            f"{sw1_name}_{result_table_file}",
            dir_path=f"{data_path}/{rascore_str}_{cluster_str}/{sw1_name}",
        )
    )
    sw2_df = load_table(
        get_file_path(
            f"{sw2_name}_{result_table_file}",
            dir_path=f"{data_path}/{rascore_str}_{cluster_str}/{sw2_name}",
        )
    )
    sw1_df = mask_equal(sw1_df, complete_col, str(True))
    sw2_df = mask_equal(sw2_df, complete_col, str(True))

    table_col_lst = [rename_col_dict[prot_class_col], rename_col_dict[match_class_col]]

    for nuc_class in nuc_class_lst:
        with st.expander(f"{nuc_class} Conformations", expanded=True):
            st.markdown(
                get_html_text(
                    {
                        nuc_class: nuc_color_dict[nuc_class],
                        " Conformations": "#31333F",
                    },
                    font_size="x-large",
                    font_weight="bold",
                ),
                unsafe_allow_html=True,
            )

            sw1_col, sw2_col = st.columns(2)

            nuc_df = mask_equal(df, nuc_class_col, nuc_class)

            for loop_name, loop_resids in loop_resid_dict.items():

                if loop_name == sw1_name:
                    cluster_df = sw1_df.copy(deep=True)
                    loop_col = sw1_col

                elif loop_name == sw2_name:
                    cluster_df = sw2_df.copy(deep=True)
                    loop_col = sw2_col


                loop_df = mask_equal(nuc_df, loop_name, [x for x in lst_col(nuc_df, loop_name,unique=True) if outlier_name not in x and disorder_name not in x])

                conf_lst = get_col_most_common(loop_df, loop_name)

                if len(conf_lst) > 1:
                    conf_str = f"{loop_name} Conformations ({len(conf_lst)} in Total)"
                else:
                    conf_str = f"{loop_name} Conformation (Only 1)"

                loop_col.markdown(f"##### {conf_str}")

                loop_conf = loop_col.selectbox("Conformation Name", conf_lst)

                conf_df = mask_equal(loop_df,loop_name,loop_conf)

                pdb_id = loop_col.selectbox(
                    "PDB ID",
                    [
                        x.upper()
                        for x in lst_col(conf_df, pdb_id_col)
                        if x in lst_col(cluster_df, pdb_id_col)
                    ],
                )

                pdb_code = pdb_id[:4].lower()
                chainid = pdb_id[4:5]

                pdb_df = mask_equal(conf_df, pdb_id_col, f"{pdb_code}{chainid}")

                if loop_name == sw1_name:
                    sw1_color = conf_color_dict[loop_name][loop_conf]
                    sw2_color = "white"
                    stick_resids = 32
                elif loop_name == sw2_name:
                    sw1_color = "white"
                    sw2_color = conf_color_dict[loop_name][loop_conf]
                    stick_resids = 71

                show_st_structure(pdb_df,
                    stick_resids=stick_resids,
                    label_resids=True, 
                    zoom_resids=loop_resids,
                    sw1_color=sw1_color,
                    sw2_color=sw2_color,
                    zoom=1.5,
                    width=400,
                    height=300,
                    st_col=loop_col)

                for table_col in table_col_lst:
                    table_df = (
                        pd.pivot_table(
                            data=rename_st_cols(
                                mask_equal(loop_df, loop_name, loop_conf)
                            ),
                            index=table_col,
                            columns=rename_col_dict[gene_class_col],
                            values=rename_col_dict[pdb_id_col],
                            aggfunc="nunique",
                            margins=True,
                        )
                        .fillna("")
                    )

                    for col in list(table_df.columns):
                        table_df[col] = table_df[col].map(str)
                        table_df = fix_col(table_df, col)

                    table_df = reorder_st_cols(table_df,reverse_col_dict[table_col],gene_class_col)
                    table_df = table_df.reset_index()

                    show_st_table(table_df, st_col=loop_col)

    write_st_end()


# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import numpy as np
import matplotlib as mpl
import seaborn as sns

from .table import make_dict
from .lst import move_end_lst

white_hex = "#ffffff"
black_hex = "#000000"

blue_hex = "#1f77b4"
orange_hex = "#ff7f0e"
green_hex = "#2ca02c"
red_hex = "#d62728"
purple_hex = "#9467bd"
brown_hex = "#8c564b"
pink_hex = "#e377c2"
gray_hex = "#7f7f7f"
olive_hex = "#bcbd22"
cyan_hex = "#17becf"

light_blue_hex = "#aec7e8"
light_orange_hex = "#ffbb78"
light_green_hex = "#98df8a"
light_red_hex = "#ff9896"
light_purple_hex = "#c5b0d5"
light_brown_hex = "#c49c94"
light_pink_hex = "#f7b6d2"
light_gray_hex = "#c7c7c7"
light_olive_hex = "#dbdb8d"
light_cyan_hex = "#9edae5"

rama_color_dict = {"A": "#e78ac3", "B": "#8da0cb", "L": "#fc8d62", "E": "#66c2a5"}


def change_hex_alpha(hex_color, alpha):

    return tuple((1.0 - alpha) + np.array(mpl.colors.hex2color(hex_color)) * alpha)


def get_palette_hex_lst(palette, total=None):

    if total is None:
        palette = sns.color_palette(palette)
    else:
        palette = sns.color_palette(palette, total)

    hex_lst = palette.as_hex()

    return hex_lst


def get_hex(color):

    return mpl.colors.to_hex(color)


def get_rgb(color):

    return mpl.colors.to_rgb(color)


def get_hex_lst(color_lst):

    for idx, color in enumerate(color_lst):
        color_lst[idx] = get_hex(color)

    return color_lst


def get_rgb_lst(color_lst):

    for idx, color in enumerate(color_lst):
        color_lst[idx] = get_rgb(color)

    return color_lst


def get_lst_colors(
    label_lst,
    palette=None,
    return_rgb=False,
    return_dict=False,
    alpha=None,
):

    if type(palette) != dict:
        total = len(label_lst)

        if palette is None:

            if total <= 10:

                color_lst = get_palette_hex_lst("tab10")

            elif total <= 20:

                color_lst = get_palette_hex_lst("tab20")

            elif total <= 40:

                color_lst_1 = get_palette_hex_lst("tab20c")
                color_lst_2 = get_palette_hex_lst("tab20b")
                color_lst = color_lst_1 + color_lst_2

            elif total > 40:

                color_lst = get_palette_hex_lst("rainbow", total)

        else:
            if type(palette) == str:
                color_lst = get_palette_hex_lst(palette, total)
            else:
                color_lst = get_hex_lst(palette)

        colors = color_lst[:total]

        if "Noise" in label_lst or "None" in label_lst:
            label_lst = move_end_lst(label_lst, ["Noise", "None"])

            colors[-1] = gray_hex

            if "Noise" in label_lst and "None" in label_lst:
                colors[-2] = gray_hex

        if alpha is not None:
            colors = [change_hex_alpha(x, alpha) for x in colors]

        if return_rgb is True:
            colors = get_rgb_lst(colors)

        if return_dict is True:
            colors = make_dict(label_lst, colors)
    else:
        colors = palette

    return colors
